@@ -10,6 +10,11 @@ function parse (bgeom) {
     const filename = 'bgeom';
     FS.createDataFile('/', filename, bgeom, true, true);
     const tesselator = new Module.Tesselator(filename);
+    const _bbox = tesselator.bbox();
+    const bbox = [
+        [_bbox.getXMin(), _bbox.getYMin(), _bbox.getZMin()],
+        [_bbox.getXMax(), _bbox.getYMax(), _bbox.getZMax()]
+    ]
     const ii = tesselator.trianglesSize();
 
     for (let i = 0; i < ii; i++) {
@@ -73,7 +78,10 @@ function parse (bgeom) {
 
     Module.destroy(tesselator);
     FS.unlink('/' + filename);
-    return triangleSets;
+    return {
+        triangleSets,
+        bbox
+    };
 }
 
 Module['parse'] = parse;
