@@ -2,18 +2,16 @@ import '../../css/widget.css';
 import {
     DOMWidgetView, WidgetView
 } from '@jupyter-widgets/base';
-import * as THREE from 'three';
 import decoder from './decoder';
-import { PGLControls, LsystemControls } from './controls';
-import { OrbitControls } from 'three/examples/jsm/controls/OrbitControls';
+import { PGLControls, LsystemControls, PGLProgress } from './controls';
 import {
     IScene,
     IPGLControlsState,
     ILsystemControlsState,
     ILsystemScene,
-    ITaskResult
+    ITaskResult, IPGLProgressState
 } from './interfaces';
-import { disposeScene, meshify } from './utilities';
+import { THREE, disposeScene, meshify } from './utilities';
 import { SCALES, LsystemUnit } from './consts';
 
 export class PGLWidgetView extends DOMWidgetView {
@@ -33,11 +31,14 @@ export class PGLWidgetView extends DOMWidgetView {
 
     sizeDisplay: number[];
     sizeWorld: number;
+    isMagic: boolean;
 
     pglControls: PGLControls = null;
     pglControlsState: IPGLControlsState = null;
     pglControlsEl: HTMLDivElement = null;
 
+    pglProgressState: IPGLProgressState = null;
+
     disposables: THREE.Scene[] = [];
     isDetached = false;
 
@@ -45,6 +46,7 @@ export class PGLWidgetView extends DOMWidgetView {
         super.initialize(parameters);
         this.sizeDisplay = parameters.model.attributes['size_display'];
         this.sizeWorld = parameters.model.attributes['size_world'];
+        this.isMagic = parameters.model.attributes['is_magic'];
     }
 
     render() {
@@ -67,7 +69,7 @@ export class PGLWidgetView extends DOMWidgetView {
         this.containerEl = document.createElement('div');
         const canvas = document.createElement('canvas');
         canvas.width = width;
-        canvas.height = height
+        canvas.height = height;
         const context = (canvas.getContext('webgl2', { alpha: false }) ||
             canvas.getContext('experimental-webgl', { alpha: false }) ||
             canvas.getContext('webgl', { alpha: false })) as WebGLRenderingContext;
@@ -85,6 +87,12 @@ export class PGLWidgetView extends DOMWidgetView {
             return false;
         });
 
+        const progressEl = document.createElement('div');
+        progressEl.setAttribute('class', 'pgl-jupyter-pgl-widget-progress');
+        this.containerEl.appendChild(progressEl);
+        const pglProgress = new PGLProgress({ busy: 0 }, progressEl);
+        this.pglProgressState = pglProgress.state;
+
         const x_size = this.sizeWorld;
         const y_size = this.sizeWorld;
         const z_size = this.sizeWorld;
@@ -150,7 +158,7 @@ export class PGLWidgetView extends DOMWidgetView {
         this.renderer.shadowMap.type = THREE.PCFSoftShadowMap;
         this.renderer.render(this.scene, this.camera);
 
-        this.orbitControl = new OrbitControls(this.camera, this.renderer.domElement);
+        this.orbitControl = new THREE.OrbitControls(this.camera, this.renderer.domElement);
         this.orbitControl.enableZoom = true;
         this.orbitControl.addEventListener('change', () => {
             this.renderer.render(this.scene, this.camera)
@@ -175,6 +183,9 @@ export class PGLWidgetView extends DOMWidgetView {
                         if (!this.pglControlsState.autoRotate) {
                             window.cancelAnimationFrame(id)
                         }
+                        const [x, y, z] = this.camera.position.toArray();
+                        console.log(x, y, Math.sqrt(Math.pow(Math.abs(x), 2) + Math.pow(Math.abs(y), 2)));
+
                         this.orbitControl.update();
                         this.renderer.render(this.scene, this.camera);
                     };
@@ -265,7 +276,7 @@ export class PGLWidgetView extends DOMWidgetView {
             const [width, height] = this.sizeDisplay;
             this.resizeDisplay(width, height);
         });
-        this.listenTo(this.model, 'change:size_world', this.resizeWorld);
+        // this.listenTo(this.model, 'change:size_world', this.resizeWorld);
         this.containerEl.onfullscreenchange = () => {
             this.pglControlsState.fullscreen = (document.fullscreenElement === this.containerEl);
             if (this.pglControlsState.fullscreen) {
@@ -297,7 +308,7 @@ export class PGLWidgetView extends DOMWidgetView {
                 }
                 break;
             case 'after-attach':
-                if (this.el.closest('.jp-LinkedOutputView')) {
+                if (this.el.closest('.jp-LinkedOutputView') && !this.isMagic) {
                     this.pWidget.addClass('pgl-resizable');
                     this.containerEl.classList.add('pgl-resizable');
                     this.isDetached = true;
@@ -313,8 +324,28 @@ export class PGLWidgetView extends DOMWidgetView {
         }
     }
 
-    resizeWorld() {
-        const size = this.model.get('size_world');
+    // resizeWorld() {
+    //     const size = this.model.get('size_world');
+    // }
+
+    fitBounds(bbox: number[][]) {
+
+        this.orbitControl.target = new THREE.Vector3(
+            0,
+            (bbox[0][1] + bbox[1][1]) / 2,
+            (bbox[0][2] + bbox[1][2]) / 2
+        );
+
+        this.camera.position.set(
+            Math.max((bbox[1][1] - bbox[0][1]), (bbox[1][2] - bbox[0][2])) * 2,
+            (bbox[0][1] + bbox[1][1]) / 2,
+            (bbox[0][2] + bbox[1][2]) / 2
+        );
+
+        this.camera.updateProjectionMatrix();
+        this.renderer.render(this.scene, this.camera);
+        this.orbitControl.update();
+
     }
 
     remove() {
@@ -348,7 +379,7 @@ export class SceneWidgetView extends PGLWidgetView {
                 decoder.decode({ data: data.buffer, userData: { position, scale, id } })
                     .then(result => {
                         const scene = new THREE.Scene();
-                        const { results, userData: { position, scale, id } } = result;
+                        const { geoms: results, userData: { position, scale, id } } = result;
                         const [x, y, z] = position;
 
                         if (results.length > 0) {
@@ -382,10 +413,12 @@ export class SceneWidgetView extends PGLWidgetView {
 
 export class LsystemWidgetView extends PGLWidgetView {
 
-    unit: LsystemUnit = LsystemUnit.M;
+    unit: LsystemUnit = LsystemUnit.NONE;
     controls: LsystemControls = null;
     queue: {[key:number]: ITaskResult} = {};
     no = 0;
+    in = 0;
+    out = 0;
 
     initialize(parameters: WidgetView.InitializeParameters) {
         super.initialize(parameters);
@@ -484,7 +517,11 @@ export class LsystemWidgetView extends PGLWidgetView {
 
         this.unit = this.model.get('unit');
         const scene = this.model.get('scene') as ILsystemScene;
-        this.decode(scene);
+        try {
+            this.decode(scene);
+        } catch (err) {
+            console.log(err);
+        }
 
         this.listenTo(this.model, 'change:unit', () => {
             this.unit = this.model.get('unit')
@@ -509,10 +546,10 @@ export class LsystemWidgetView extends PGLWidgetView {
             const decoded = this.queue[no];
             if (decoded) {
                 delete this.queue[no];
-                this.setScene(decoded.userData.step, meshify(decoded.results, {
+                this.setScene(decoded.userData.step, meshify(decoded.geoms, {
                     flatShading: this.pglControlsState.flatShading,
                     wireframe: this.pglControlsState.wireframe,
-                }));
+                }), decoded.bbox);
                 this.controls.state.derivationStep = decoded.userData.step;
                 if (this.controls.state.busy > 0) {
                     this.controls.state.busy--;
@@ -530,17 +567,18 @@ export class LsystemWidgetView extends PGLWidgetView {
     }
 
     decode(scene: ILsystemScene) {
+        this.pglProgressState.busy = 1 - (this.out / ++this.in);
         decoder.decode({ data: scene.data.buffer, userData: { step: scene.derivationStep, no: this.no++ } }, this.model.model_id)
             .then(res => {
                 // TODO: use increasing number for a seq. of tasks and not step
-                const { results, userData: { step, no } } = res;
+                const { geoms, bbox, userData: { step, no } } = res;
                 if (this.controls.state.animate && step - this.controls.state.derivationStep > 1) {
                     this.queue[no] = res;
                 } else {
-                    this.setScene(step, meshify(results, {
+                    this.setScene(step, meshify(geoms, {
                         flatShading: this.pglControlsState.flatShading,
                         wireframe: this.pglControlsState.wireframe,
-                    }));
+                    }), bbox);
                     this.controls.state.derivationStep = step;
                     if (this.controls.state.busy > 0) {
                         this.controls.state.busy--;
@@ -565,14 +603,22 @@ export class LsystemWidgetView extends PGLWidgetView {
             });
     }
 
-    setScene(step: number, meshs: Array<THREE.Mesh | THREE.InstancedMesh>, position = [0, 0, 0]) {
-
+    setScene(step: number, meshs: Array<THREE.Mesh | THREE.InstancedMesh>, bbox=null, position=[0, 0, 0]) {
+        this.pglProgressState.busy = 1 - (++this.out / this.in);
+        if (this.in === this.out) {
+            this.in = this.out = 0;
+        }
         const currentScene = new THREE.Scene();
         const [x, y, z] = position;
         const scale = SCALES[this.unit];
         currentScene.scale.multiplyScalar(scale);
         currentScene.position.set(x, y, z);
-        currentScene.add(...meshs);
+        if (meshs.length) {
+            if (bbox && this.unit === LsystemUnit.NONE) {
+                this.fitBounds(bbox);
+            }
+            currentScene.add(...meshs);
+        }
         currentScene.userData = { step };
         currentScene.name = 'lsystem';
 
